@@ -9,17 +9,26 @@ from __future__ import absolute_import
 
 import logging
 import os
+import platform
 import sys
 
 import click
 import kaptan
 from click.exceptions import FileError
 
-from libtmux.common import has_gte_version, has_minimum_version, which
+from libtmux.common import (
+    has_gte_version,
+    has_minimum_version,
+    which,
+    get_version,
+    tmux_cmd,
+)
 from libtmux.exc import TmuxCommandNotFound
 from libtmux.server import Server
 
-from . import config, exc, log, util
+from libtmux import __version__ as libtmux_version
+
+from . import config, exc, log, util, __file__ as tmuxp_path
 from .__about__ import __version__
 from ._compat import PY3, PYMINOR, string_types
 from .workspacebuilder import WorkspaceBuilder, freeze
@@ -33,6 +42,18 @@ def get_cwd():
     return os.getcwd()
 
 
+def tmuxp_echo(message=None, log_level='INFO', style_log=False, **click_kwargs):
+    """
+    Combines logging.log and click.echo
+    """
+    if style_log:
+        logger.log(log.LOG_LEVELS[log_level], message)
+    else:
+        logger.log(log.LOG_LEVELS[log_level], click.unstyle(message))
+
+    click.echo(message, **click_kwargs)
+
+
 def get_config_dir():
     """
     Return tmuxp configuration directory.
@@ -237,8 +258,8 @@ def scan_config_argument(ctx, param, value, config_dir=None):
         config_dir = config_dir()
 
     if not config:
-        click.echo("Enter at least one CONFIG")
-        click.echo(ctx.get_help(), color=ctx.color)
+        tmuxp_echo("Enter at least one CONFIG")
+        tmuxp_echo(ctx.get_help(), color=ctx.color)
         ctx.exit()
 
     if isinstance(value, string_types):
@@ -348,11 +369,14 @@ def scan_config(config, config_dir=None):
             ]
 
             if len(candidates) > 1:
-                click.secho(
-                    'Multiple .tmuxp.{yml,yaml,json} configs in %s' % dirname(config),
-                    fg="red",
+                tmuxp_echo(
+                    click.style(
+                        'Multiple .tmuxp.{yml,yaml,json} configs in %s'
+                        % dirname(config),
+                        fg="red",
+                    )
                 )
-                click.echo(
+                tmuxp_echo(
                     click.wrap_text(
                         'This is undefined behavior, use only one. '
                         'Use file names e.g. myproject.json, coolproject.yaml. '
@@ -399,6 +423,7 @@ def load_workspace(
     config_file,
     socket_name=None,
     socket_path=None,
+    tmux_config_file=None,
     new_session_name=None,
     colors=None,
     detached=False,
@@ -496,6 +521,11 @@ def load_workspace(
     # get the canonical path, eliminating any symlinks
     config_file = os.path.realpath(config_file)
 
+    tmuxp_echo(
+        click.style('[Loading] ', fg='green')
+        + click.style(config_file, fg='blue', bold=True)
+    )
+
     # kaptan allows us to open a yaml or json file as a dict
     sconfig = kaptan.Kaptan()
     sconfig = sconfig.import_config(config_file).get()
@@ -508,7 +538,8 @@ def load_workspace(
     sconfig = config.trickle(sconfig)
 
     t = Server(  # create tmux server object
-        socket_name=socket_name, socket_path=socket_path, colors=colors
+        socket_name=socket_name, socket_path=socket_path,
+        config_file=tmux_config_file, colors=colors,
     )
 
     which('tmux')  # raise exception if tmux not found
@@ -516,7 +547,7 @@ def load_workspace(
     try:  # load WorkspaceBuilder object for tmuxp config / tmux server
         builder = WorkspaceBuilder(sconf=sconfig, server=t)
     except exc.EmptyConfigException:
-        click.echo('%s is empty or parsed no config data' % config_file, err=True)
+        tmuxp_echo('%s is empty or parsed no config data' % config_file, err=True)
         return
 
     session_name = sconfig['session_name']
@@ -536,11 +567,6 @@ def load_workspace(
         return
 
     try:
-        click.echo(
-            click.style('[Loading] ', fg='green')
-            + click.style(config_file, fg='blue', bold=True)
-        )
-
         builder.build()  # load tmux session via workspace builder
 
         if 'TMUX' in os.environ:  # tmuxp ran from inside tmux
@@ -577,8 +603,8 @@ def load_workspace(
     except exc.TmuxpException as e:
         import traceback
 
-        click.echo(traceback.format_exc(), err=True)
-        click.echo(e, err=True)
+        tmuxp_echo(traceback.format_exc(), err=True)
+        tmuxp_echo(e, err=True)
 
         choice = click.prompt(
             'Error loading workspace. (k)ill, (a)ttach, (d)etach?',
@@ -588,7 +614,7 @@ def load_workspace(
 
         if choice == 'k':
             builder.session.kill_session()
-            click.echo('Session killed.')
+            tmuxp_echo('Session killed.')
         elif choice == 'a':
             if 'TMUX' in os.environ:
                 builder.session.switch_client()
@@ -603,7 +629,7 @@ def load_workspace(
 @click.group(context_settings={'obj': {}})
 @click.version_option(__version__, '-V', '--version', message='%(prog)s %(version)s')
 @click.option(
-    '--log_level',
+    '--log-level',
     default='INFO',
     help='Log level (DEBUG, INFO, WARNING, ERROR, CRITICAL)',
 )
@@ -616,12 +642,12 @@ def cli(log_level):
     try:
         has_minimum_version()
     except TmuxCommandNotFound:
-        click.echo('tmux not found. tmuxp requires you install tmux first.')
+        tmuxp_echo('tmux not found. tmuxp requires you install tmux first.')
         sys.exit()
     except exc.TmuxpException as e:
-        click.echo(e, err=True)
+        tmuxp_echo(e, err=True)
         sys.exit()
-    setup_logger(level=log_level.upper())
+    setup_logger(logger=logger, level=log_level.upper())
 
 
 def setup_logger(logger=None, level='INFO'):
@@ -640,12 +666,12 @@ def setup_logger(logger=None, level='INFO'):
         logger = logging.getLogger()
 
     if not logger.handlers:  # setup logger handlers
-        channel = logging.StreamHandler()
-        channel.setFormatter(log.DebugLogFormatter())
-
+        # channel = logging.StreamHandler()
+        # channel.setFormatter(log.DebugLogFormatter())
         # channel.setFormatter(log.LogFormatter())
+
         logger.setLevel(level)
-        logger.addHandler(channel)
+        # logger.addHandler(channel)
 
 
 def startup(config_dir):
@@ -848,6 +874,7 @@ def command_freeze(session_name, socket_name, socket_path, force):
 @click.argument('config', type=ConfigPath(exists=True), nargs=-1)
 @click.option('-S', 'socket_path', help='pass-through for tmux -S')
 @click.option('-L', 'socket_name', help='pass-through for tmux -L')
+@click.option('-f', 'tmux_config_file', help='pass-through for tmux -f')
 @click.option('-s', 'new_session_name', help='start new session with new session name')
 @click.option('--yes', '-y', 'answer_yes', help='yes', is_flag=True)
 @click.option(
@@ -866,15 +893,18 @@ def command_freeze(session_name, socket_name, socket_path, force):
     flag_value=88,
     help='Like -2, but indicates that the terminal supports 88 colours.',
 )
+@click.option('--log-file', help='File to log errors/output to')
 def command_load(
     ctx,
     config,
     socket_name,
     socket_path,
+    tmux_config_file,
     new_session_name,
     answer_yes,
     detached,
     colors,
+    log_file,
 ):
     """Load a tmux workspace from each CONFIG.
 
@@ -899,10 +929,15 @@ def command_load(
     detached mode.
     """
     util.oh_my_zsh_auto_title()
+    if log_file:
+        logfile_handler = logging.FileHandler(log_file)
+        logfile_handler.setFormatter(log.LogFormatter())
+        logger.addHandler(logfile_handler)
 
     tmux_options = {
         'socket_name': socket_name,
         'socket_path': socket_path,
+        'tmux_config_file': tmux_config_file,
         'new_session_name': new_session_name,
         'answer_yes': answer_yes,
         'colors': colors,
@@ -910,8 +945,8 @@ def command_load(
     }
 
     if not config:
-        click.echo("Enter at least one CONFIG")
-        click.echo(ctx.get_help(), color=ctx.color)
+        tmuxp_echo("Enter at least one CONFIG")
+        tmuxp_echo(ctx.get_help(), color=ctx.color)
         ctx.exit()
 
     if isinstance(config, string_types):
@@ -953,7 +988,7 @@ def import_config(configfile, importfunc):
     else:
         sys.exit('Unknown config format.')
 
-    click.echo(
+    tmuxp_echo(
         newconfig + '---------------------------------------------------------------'
         '\n'
         'Configuration import does its best to convert files.\n'
@@ -975,9 +1010,9 @@ def import_config(configfile, importfunc):
         buf.write(newconfig)
         buf.close()
 
-        click.echo('Saved to %s.' % dest)
+        tmuxp_echo('Saved to %s.' % dest)
     else:
-        click.echo(
+        tmuxp_echo(
             'tmuxp has examples in JSON and YAML format at '
             '<http://tmuxp.git-pull.com/examples.html>\n'
             'View tmuxp docs at <http://tmuxp.git-pull.com/>'
@@ -1054,3 +1089,66 @@ def command_ls():
             if os.path.isdir(f) or ext not in VALID_CONFIG_DIR_FILE_EXTENSIONS:
                 continue
             print(stem)
+
+
+@cli.command(name='debug-info', short_help='Print out all diagnostic info')
+def command_debug_info():
+    """
+    Print debug info to submit with Issues.
+    """
+
+    def prepend_tab(strings):
+        """
+        Prepend tab to strings in list.
+        """
+        return list(map(lambda x: '\t%s' % x, strings))
+
+    def output_break():
+        """
+        Generate output break.
+        """
+        return '-' * 25
+
+    def format_tmux_resp(std_resp):
+        """
+        Format tmux command response for tmuxp stdout.
+        """
+        return '\n'.join(
+            [
+                '\n'.join(prepend_tab(std_resp.stdout)),
+                click.style('\n'.join(prepend_tab(std_resp.stderr)), fg='red'),
+            ]
+        )
+
+    output = [
+        output_break(),
+        'environment:\n%s'
+        % '\n'.join(
+            prepend_tab(
+                [
+                    'dist: %s' % platform.platform(),
+                    'arch: %s' % platform.machine(),
+                    'uname: %s' % '; '.join(platform.uname()[:3]),
+                    'version: %s' % platform.version(),
+                ]
+            )
+        ),
+        output_break(),
+        'python version: %s' % ' '.join(sys.version.split('\n')),
+        'system PATH: %s' % os.environ['PATH'],
+        'tmux version: %s' % get_version(),
+        'libtmux version: %s' % libtmux_version,
+        'tmuxp version: %s' % __version__,
+        'tmux path: %s' % which('tmux'),
+        'tmuxp path: %s' % tmuxp_path,
+        'shell: %s' % os.environ['SHELL'],
+        output_break(),
+        'tmux sessions:\n%s' % format_tmux_resp(tmux_cmd('list-sessions')),
+        'tmux windows:\n%s' % format_tmux_resp(tmux_cmd('list-windows')),
+        'tmux panes:\n%s' % format_tmux_resp(tmux_cmd('list-panes')),
+        'tmux global options:\n%s' % format_tmux_resp(tmux_cmd('show-options', '-g')),
+        'tmux window options:\n%s'
+        % format_tmux_resp(tmux_cmd('show-window-options', '-g')),
+    ]
+
+    tmuxp_echo('\n'.join(output))
